/*
	Main.cpp
    Written by Saeed Amizadeh and Amir Nayyeri (PGen Project)

	Command-Line Parser (Console Interface for CParser)
*/

#include <fstream.h>
#include <iostream>
#include "headers.h"
#include "scg.h"
#include "parse.h"

void main(int argc, char* argv[])
{
	fstream ParseTable,outf,inf;

	if ((argc != 3) && (argc != 4))
	{
		cout << "Usage:\tParser File1 File2 [File3]\n\n"
				"File1:\tInput File\n"
				"File2:\tOutput File\n"
				"File3:\tParse Table Resource (.pt File Generated by PGen)\n"
				"\tDefault for File3 is Grammar.pt .\n";
		
		return;
	}
	if (argc == 3)
		ParseTable.open("Grammar.pt",ios::in | ios::binary);
	else
		ParseTable.open(argv[3],ios::in | ios::binary);
	if(!ParseTable.good())
	{
		cout << "Unable to open grammer file." << endl;
		return;
	}
	outf.open(argv[2],ios::out);
	if(!outf.good())
	{
		cout << "Unable to open " << argv[2] << endl;
		return;
	}
	inf.open(argv[1],ios::in);
	if(!inf.good())
	{
		cout << "Unable to open " << argv[1] << endl;
		return;
	}

	try
	{
		CParser MyParser(ParseTable, inf, outf);
		MyParser.Parse();
		cout << "\a Success !!!!!!!\n";
	}
	catch(CParseErrorException pe)
	{
		cout << " Illegal syntax!!!" << endl;
		
		// Add your extra code here for powerful exception handling (Optional)
	}
	catch(CInvalidFormatException ife)
	{
		cout << " Invalid parse table file format!!!" << endl;
		
		// Add your extra code here for powerful exception handling (Optional)
	}
	catch(Except myexcept)
	{
		std::cout << myexcept.message << std::endl;
	}
	
	ParseTable.close();
	outf.close();
	inf.close();
}
