%{
#include "y.tab.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
int linenum = 0;
int tokenNumber;
int ival = 0;
float rval = 0;
void yyerror( char *, char *);
%}
/****************************************************/
white_space	[ \t]*
digit		[0-9]
hex_digit	[0-9A-Fa-f]
oct_digit	[0-7]
alpha		[_A-Za-z]
identifier	{alpha}({alpha}|{digit})*
isuffix		[uU]|[lL]|("ll"|"LL")|("lu"|"LU"|"Lu"|"lU")|("llu"|"LLU"|"LLu"|"llU")
decint		[+-]?{digit}+{isuffix}?
octint		"0"{oct_digit}*{isuffix}?
hexint		"0"[xX]{hex_digit}+{isuffix}?
dsuffix		[fF]|[lL]
i		{digit}+
exponent	[eE][+-]?{i}
floating_point	(({i}\.{i}?|{i}?\.{i}){exponent}?)|({i}{exponent})
special		\\([nat\?vbrf\'\\\"]|([0-7]{1,3})|(x{hex_digit}{1,2}))
char_const	\'([^\\\n]|{special})\'
%%

"/*" {
     int c;
     while( ( c = yyinput() ) != 0 )
     {
         if ( c == '\n' ) linenum++;
         else if ( c == '*' )
	 {
	     if ( (c = yyinput()) == '/' ) break;
	     else unput(c);
	 }
     }
}

\"  {
	char cur = 0;
        char prev = 0;
	char anocot = 0;
        int i = 1;
        char buf [100];
        strcpy(buf,"\"");

        while ( (cur=yyinput()) != 0 )
	{
	    buf[i++] = cur;

	    if ( cur == '\n' ) yyerror("unterminated string constant", yytext);
	    else if ( cur == '"' )
	    {
		if ( prev != '\\' )
		{
		    buf[i] = '\0';
		    strcpy( yytext, buf);
		    return(STRING);
		}
	    }

	    prev = cur;
        }
    }

"asm"			{return ASM;}
"default"		{return DEFAULT;}
"for"			{return FOR;}
"short"			{return SHORT;}
"union"			{return UNION;}
"auto"			{return AUTO;}
"do"			{return DO;}
"goto"			{return GOTO;}
"signed"		{return SIGNED;}
"unsigned"		{return UNSIGNED;}
"break"			{return BREAK;}
"double"		{return DOUBLE;}
"if"			{return IF;}
"void"			{return VOID;}
"case"			{return CASE;}
"else"			{return ELSE;}
"int"			{return INT;}
"static"		{return STATIC;}
"volatile"		{return VOLATILE;}
"char"			{return CHAR;}
"enum"			{return ENUM;}
"long"			{return LONG;}
"struct"		{return STRUCT;}
"while"			{return WHILE;}
"const"			{return CONST;}
"extern"		{return EXTERN;}
"register"		{return REGISTER;}
"switch"		{return SWITCH;}
"continue"		{return CONTINUE;}
"float"			{return FLOAT;}
"return"		{return RETURN;}
"typedef"		{return TYPEDEF;}
"printInt"		{return PRINTINT;}
"printChar"		{return PRINTCHAR;}
"printString"	{return PRINTSTRING;}

{decint}		{
			    sscanf(yytext, "%d", &ival);
			    return DECINT;
			}

{octint}		{
			    sscanf(yytext, "%o", &ival);
			    return OCTINT;
			}

{hexint}		{
			    sscanf(yytext, "%x", &ival);
			    return HEXINT;
			}

{floating_point}	{
			    sscanf(yytext, "%f", &rval);
			    return FLOATINGPOINT;
			}

{char_const}	{return CHARCONSTANT;}
{identifier}	{return ID;}
">>="			{ return(RIGHT_ASSIGN); }
"<<="			{ return(LEFT_ASSIGN); }
"+="			{ return(ADD_ASSIGN); }
"-="			{ return(SUB_ASSIGN); }
"*="			{ return(MUL_ASSIGN); }
"/="			{ return(DIV_ASSIGN); }
"%="			{ return(MOD_ASSIGN); }
"&="			{ return(AND_ASSIGN); }
"^="			{ return(XOR_ASSIGN); }
"|="			{ return(OR_ASSIGN); }
">>"			{ return(SHR); }
"<<"			{ return(SHL); }
"++"			{ return(INC); }
"--"			{ return(DEC); }
"->"			{ return(PTR); }
"&&"			{ return(AND); }
"||"			{ return(OR); }
"<="			{ return(LE); }
">="			{ return(GE); }
"=="			{ return(EQ); }
"!="			{ return(NE); }
";"				{ return(SEMICOLON); }
"{"				{ return(LBRACE); }
"}"				{ return(RBRACE); }
","				{ return(COMMA); }
":"				{ return(COLON); }
"="				{ return(ASSIGN); }
"("				{ return(LPARENTHESIS); }
")"				{ return(RPARENTHESIS); }
"["				{ return(LBRACKET); }
"]"				{ return(RBRACKET); }
"."				{ return(DOT); }
"&"				{ return(AMPERSAND); }
"!"				{ return(EXCLAMATION); }
"~"				{ return(TILDE); }
"-"				{ return(DASH); }
"+"				{ return(PLUS); }
"*"				{ return(ASTERISK); }
"/"				{ return(SLASH); }
"%"				{ return(PERCENT); }
"<"				{ return(LESS); }
">"				{ return(GREATER); }
"^"				{ return(CARET); }
"|"				{ return(VERTICAL_BAR); }
"?"				{ return(QUESTION); }
{white_space}		;
\n			linenum++;
.			{yyerror("Illegal input", yytext);}
%%
/****************************************************/
int endOfFile = 0;

int yywrap() { endOfFile = 1; return 1; }

char tokenNames[][20] =
    {
	"", "id", "ASM", "DEFAULT", "for", "SHORT", "UNION", "AUTO", "DO", "GOTO", "SIGNED"
	, "UNSIGNED", "BREAK", "DOUBLE", "if", "VOID", "CASE", "else", "int", "STATIC"
	, "VOLATILE", "char", "ENUM", "LONG", "STRUCT", "while", "CONST", "EXTERN", "REGISTER"
	, "SWITCH", "CONTINUE", "FLOAT", "return", "TYPEDEF", "number", "OCTINT", "HEXINT"
	, "quotedChar", "quotedString", "SEPARATOR", "COMMENT", "FLOAPOING", ">>=", "<<="
	, "+=", "-=", "*=", "/=", "MOD_ASSIGN", "&="
	, "XOR_ASSIGN", "|=", ">>", "<<", "++", "--", "PTR_OP"
	, "&&", "||", "<=", ">=", "==", "!=", ";" , "{"
	, "}", ",", "COLON", "=", "(", ")", "["
	, "]", "DOT", "&", "!", "TILDE", "-", "+", "*"
	, "/", "PERCENT", "<", ">", "CARET", "|"
	, "QUESTION", "printInt", "printChar", "printString"
};

void yyerror( char* message, char* mytoki)
{
    fprintf( stderr, "Error: \"%s\" in line %d. Token = %s\n",
        message,linenum,mytoki);
    exit(1);
}

/*
int main( int argc, char** argv)
{
        ++argv, --argc;  // skip over program name
        if ( argc > 0 )
                yyin = fopen( argv[0], "r" );
        else
                yyin = stdin;
        tokenNumber = yylex();

        printf("\n#\tType\tLexeme\n-----------------------------------------------------------");
        while (!endOfFile)
        {
            printf("\n%d\t%s\t%s",tokenNumber, tokenNames[tokenNumber], yytext);

	    if(tokenNumber == DECINT || tokenNumber == HEXINT || tokenNumber == OCTINT)
		printf("\t----> %d", ival);
	    else if(tokenNumber == FLOATINGPOINT)
		printf("\t----> %f", rval);

            tokenNumber = yylex();
	}

	printf("\n");
	return 0;
}
*/