/*
	scg.cpp
    Written by Saeed Amizadeh and Amir Nayyeri (PGen Project)
	
	Edited by Mohsen Lesani and Amir Moghimi
	Thanks to S.A. and A.N.

	Primitive methods for Scanner and Code Generator
*/

#include "headers.h"
#include "scg.h"
#include <stdlib.h>
#include "Expression.h"

extern char tokenNames[][20];

DSCP * vect( std::vector<DSCP *> & tmp, int i)
{
	return tmp[i];
}

// GLOBAL FUNCTION  /////////////////////////////////////////////////////////////////////
std::string int2String(int what)
{
	if(what == 0)
		return "0";

	std::string retval, temp;

	while(what != 0)
	{
		temp += what - (what / 10) * 10 + '0';
		what /= 10;
	}
	
	what = temp.size();
	retval.resize(what);

	for(int a = 0;what > 0;what--,a++)
		retval[what - 1] = temp[a];

	return retval;
}


// Tree     Methods /////////////////////////////////////////////////////////////////////
Tree::~Tree()
{
	if(left != NULL)
		delete left;

	if(right != NULL)
		delete right;

	if(arguments != NULL)
	{
		int end = arguments->size();

		for(int i = 0;i < end;i++)
			delete (*arguments)[i];

		delete arguments;
	}
}

// CScanner Methods /////////////////////////////////////////////////////////////////////

CScanner::CScanner(istream& in): yyFlexLexer(&in), inputStream(in) { }
//---------------------------------------------------------------------------------------

STRING CScanner::NextToken()
{
	int index = yylex();
	//cout << tokenNames[index] << endl;
	return tokenNames[index];
}
//---------------------------------------------------------------------------------------

STRING CScanner::lexeme()
{
	return YYText();
}

// CCodeGenerator Methods /////////////////////////////////////////////////////////////////////

CCodeGenerator::CCodeGenerator(ostream& out): outputStream(out) 
{
	literalcounter = 0;
	labelcounter = 0;

	outputStream  << "; Assemby Code Generated by HN C Compiler\n";
	outputStream  << "\tpublic\t_main\n"
					 "\textrn\t_printf : near\n\n";


	anotherStream +=  "_TEXT\tsegment\tbyte\tpublic\t'CODE'\n"
					  "\tassume CS:_TEXT, DS:_DATA\n\n";

//	tempStream += "\tMOV\tAX,\tDSeg\n"
//				  "\tMOV\tDS,\tAX\n";
}

CCodeGenerator::~CCodeGenerator()
{
	int end = DataSeg.size();

	outputStream << "_DATA\tsegment\tword\tpublic\t'DATA'\n";
	outputStream << "\t_HNSS\tDB\t'%s'\n"
					"\t\tDB\t0\n";
	outputStream << "\t_HNSI\tDB\t'%d'\n"
					"\t\tDB\t0\n";
	outputStream << "\t_HNSC\tDB\t'%c'\n"
					"\t\tDB\t0\n";

	for(int i = 0;i < end;i++)
		outputStream << '\t' << DataSeg[i].c_str();

	outputStream << "_DATA\tends\n\n";
	DataSeg.resize(0);
	outputStream << anotherStream.c_str();
	anotherStream.resize(0);

	outputStream << "_main\tPROC\tNEAR\n"
					"\tPUSH\tBP\n"
					"\tMOV\tBP, SP\n"
					"\t;-----------------------------------\n";
	outputStream << tempStream.c_str()
				 << "\tcall\t__main\n"
					"\t;-----------------------------------\n"
					"\tPOP\tCX\n"
					"\tRET\n"
					"_main\tENDP\n\n"
					"_TEXT\tENDS\n\n"
					"END\n";

	end = myframe.size();

	for(i = 0;i < end;i++)
		delete myframe[i];

	end = SymbolTable.size();

	for(i = 0;i < end;i++)
		delete SymbolTable[i];

//NAEEM+++++++++++++++++++++++++++++++++++++++++++LEAVE TO E+++++++++++++++++++++++++++++
	/*
	anotherStream += "\tMOV\tAX,\t4C00H\n"
					 "\tINT\t21H\n"
					 "main\tendp\n\n";
	
	anotherStream +="printInt\tproc far\t;prints an integer number\n"
					"\tPUSH\tBP\n"
					"\tMOV\tBP, SP\n"
					"\tPUSH\tAX\n"
					"\tPUSH\tSI\n"
					"\tPUSH\tDX\n"
					"\tPUSH\tCX\n"
					"\tSUB\tSP, 5d\n"
					"\tMOV\tAX, [BP + 6]\n"
					"\tMOV\tCX, 10d\n"
					"\tXOR\tSI, SI\n"
					"PFL1:\tXOR\tDX, DX\n"
					"\tDIV\tCX\n"
					"\tMOV\tBYTE PTR [BP+SI-13], DL\n"
					"\tINC\tSI\n"
					"\tCMP\tSI, 5d\n"
					"\tJNE\tPFL1\n"
					"\t\n"
					"\tMOV\tSI, 5d\n"
					"PFL2:\tCMP\tSI, 0\n"
					"\tJNE\tPFL3\n"
					"\tJMP\tPFL4\n"
					"PFL3:\tMOV\tDL, BYTE PTR [BP+SI-14]\n"
					"\tDEC\tSI\n"
					"\tCMP\tDL, 0\n"
					"\tJNE\tPFL4\n"
					"\tJMP\tPFL2\n"
					"\t\n"
					"PFL4:\tOR\tDL, 30h\n"
					"\tMOV\tAH, 2\n"
					"\tINT\t21H\n"
					"\tCMP\tSI, 0\n"
					"\tJE\tPFL5\n"
					"\tMOV\tDL, BYTE PTR [BP+SI-14]\n"
					"\tDEC\tSI\n"
					"\tJMP\tPFL4\n"
					"\t\n"
					"PFL5:\tMOV\tAH, 2\n"
					"\tMOV\tDL, 0AH\n"
					"\tINT\t21H\n"
					"\tMOV\tDL, 0DH\n"
					"\tINT\t21H\n"
					"\t\n"
					"\tADD\tSP, 5d\n"
					"\tPOP\tCX\n"
					"\tPOP\tDX\n"
					"\tPOP\tSI\n"
					"\tPOP\tAX\n"
					"\tPOP\tBP\n"
					"\tRET\n"
					"printInt\tendp\n\n";

	anotherStream += "CSeg\tends\n\n";

	anotherStream += "DSeg\tsegment\n";

	for (int i = 0; i < 26; i++)
	{
		if (symbolTable[i])
		{
			anotherStream += "ID_";
			anotherStream += char('a' + i);
			anotherStream += "\tDW\t?\n";
		}
	}

	anotherStream += "DSeg\tends\n";
	anotherStream += "\nend\n";
*/
}
//---------------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////////////
DSCP* CCodeGenerator::findDSCP( STRING what2find)
{
	// We add an underline to the begining of a variable
	
	int end = myframe.size();

	for(int i = 0;i < end;i++)
	{
		//cout << myframe[i]->getname().c_str() << endl;
		if(myframe[i]->getname() == what2find)
			return myframe[i];
	}

	end = SymbolTable.size();
	what2find.insert( what2find.begin(), '_');

	for(i = 0;i < end;i++)
	{
		//cout << SymbolTable[i]->getname().c_str() << endl;
		if(SymbolTable[i]->getname() == what2find)
			return SymbolTable[i];
	}
	return NULL;
}

int CCodeGenerator::findDSCPinglobal( STRING what2find)
{
	int end = SymbolTable.size();

	for(int i = 0;i < end;i++)
	{
		if(SymbolTable[i]->getname() == what2find)
			return true;
	}

	return false;
}

int CCodeGenerator::findwithdifferentargs( STRING what2find)
{
	int end = funcprototypes.size();

	FuncCell mytemp( INT, what2find, myargs);

	for(int i = 0;i < end;i++)
	{
		if(funcprototypes[i] == mytemp)
			return true;
	}

	return false;
}

int CCodeGenerator::findDSCPinframe( STRING what2find)
{
	int end = myframe.size();

	for(int i = 0;i < end;i++)
	{
		if(myframe[i]->getname() == what2find)
			return true;
	}

	return false;
}

int CCodeGenerator::searchforthisinfunctionframe( STRING what2find)		
{
	int end = funcprototypes.size();

	for(int i = 0;i < end;i++)
	{
		if(funcprototypes[i].name == what2find)
			return true;
	}

	return false;
}

STRING CCodeGenerator::nextLabel()
{
	STRING StrLab = "_HNL";
	StrLab += int2String(labelcounter++);

	return StrLab;
}

STRING CCodeGenerator::nextLiteral()
{
	STRING StrLit = "_HNV";
	StrLit += int2String(literalcounter++);

	return StrLit;
}
/////////////////////////////////////////////////////////////////////////////////////////


void CCodeGenerator::Generate(const STRING& semanticToken,
								   const STRING& token, const STRING& lexeme)
{
//NAEEM+++++++++++++++++++++++++++++++++++++++++++MUST CHANGE+++++++++++++++++++++++++++++
//MAIN graph
	if (semanticToken == "@StrLit")
	{
		STRING StrLit = nextLiteral();

		STRING DatasegVar = (SStk.top()).label;
		SStk.pop();
		
		if((SStk.top()).prevtype != CHARSTAR)
			throw Except("ERR : A STRING CANNOT ASSIGN TO NONE CHAR * TYPE IDS");
		
		tempStream += "\tLEA\tAX, ";
		tempStream += StrLit.c_str();
		tempStream += "\n";
		tempStream += "\tMOV\t";

		tempStream += DatasegVar.c_str();
		tempStream += ", AX\n";

		StrLit += "\tDB\t\'";
		StrLit += STRING( lexeme, 1, lexeme.size() - 2);
		StrLit += "\'\n\t\tDB\t0\n";
		DataSeg.push_back(StrLit);

		DatasegVar += "\tDW\t0\n";
		DataSeg.push_back(DatasegVar);
	}
	else if (semanticToken == "@datasegIDPUSH")
	{
		SStk.push( SSCell(lexeme));
	}
	else if (semanticToken == "@datasegMakeSimple")
	{
		if( findDSCPinglobal( (SStk.top()).label ))
			throw Except("ERR : ID IS ALREADY DEFINED -> " + (SStk.top()).label );

		STRING DatasegVar = "_" + (SStk.top()).label;
		SStk.pop();
		Simple * simple = new Simple(DatasegVar, DatasegVar, (SStk.top()).prevtype);

		DatasegVar += "\tDW\t0\n";
		DataSeg.push_back(DatasegVar);
		SymbolTable.push_back(simple);
	}
	else if (semanticToken == "@datasegMakeSimple&popSS")
	{
		if( findDSCPinglobal( (SStk.top()).label ))
			throw Except("ERR : ID IS ALREADY DEFINED -> " + (SStk.top()).label );

		STRING DatasegVar = "_" + (SStk.top()).label;
		SStk.pop();
		Simple * simple = new Simple(DatasegVar, DatasegVar, (SStk.top()).prevtype);

		DatasegVar += "\tDW\t0\n";
		DataSeg.push_back(DatasegVar);
		SymbolTable.push_back(simple);

		SStk.pop();
	}
	else if (semanticToken == "@arrayPUSHCONST")
	{
		int firsttemp;
		sscanf(lexeme.c_str(), "%d", &firsttemp); 
		SStk.push( SSCell(firsttemp, 0));	
	}
	else if (semanticToken == "@datasegMake1DArray")
	{
		int firstdim = (SStk.top()).columnnum;
		SStk.pop();
		STRING DatasegVar = "_" + (SStk.top()).label;
		SStk.pop();

		STRING mycode = "[";
		mycode += DatasegVar;
		//mycode += "]";

		OneDArr * onedarr = new OneDArr(DatasegVar, mycode, (SStk.top()).prevtype);

		DatasegVar += "\tDW\t";
		DatasegVar += int2String( firstdim);
		DatasegVar += "\tDUP\t(?)\n";
		DataSeg.push_back(DatasegVar);
		SymbolTable.push_back(onedarr);
	}
	else if (semanticToken == "@datasegMake1DArray&popSS")
	{
		int firstdim = (SStk.top()).columnnum;
		SStk.pop();
		STRING DatasegVar = "_" + (SStk.top()).label;
		SStk.pop();

		STRING mycode = "[";
		mycode += DatasegVar;
		//mycode += "]";

		OneDArr * onedarr = new OneDArr(DatasegVar, mycode, (SStk.top()).prevtype);

		DatasegVar += "\tDW\t";
		DatasegVar += int2String( firstdim);
		DatasegVar += "\tDUP\t(?)\n";
		DataSeg.push_back(DatasegVar);
		SymbolTable.push_back(onedarr);

		SStk.pop();
	}
	else if (semanticToken == "@datasegMake2DArray")
	{
		int seconddim = (SStk.top()).columnnum;
		SStk.pop();
		int firstdim = (SStk.top()).columnnum;
		SStk.pop();
		STRING DatasegVar = "_" + (SStk.top()).label;
		SStk.pop();

		STRING mycode = "[";
		mycode += DatasegVar;
		//mycode += "]";

		TwoDArr * twodarr = new TwoDArr(DatasegVar, mycode, seconddim, (SStk.top()).prevtype);

		DatasegVar += "\tDW\t";
		DatasegVar += int2String( firstdim * seconddim);
		DatasegVar += "\tDUP\t(?)\n";
		DataSeg.push_back(DatasegVar);
		SymbolTable.push_back(twodarr);
	}
	else if (semanticToken == "@datasegMakeSimpleINIT")
	{
		if( findDSCPinglobal( (SStk.top()).label ))
			throw Except("ERR : ID IS ALREADY DEFINED -> " + (SStk.top()).label );

		STRING DatasegVar = "_" + (SStk.top()).label;
		SStk.pop();
		Simple * simple = new Simple(DatasegVar, DatasegVar, (SStk.top()).prevtype);

		SStk.push(DatasegVar);
		SymbolTable.push_back(simple);
	}
	else if (semanticToken == "@arrayPOPTYPE")
	{
		SStk.pop();
	}
	else if (semanticToken == "@HOSSEIN@INITE")
	{
		//expressioncodder( tempStream);
		Expression* e = new Expression( root, this);
		delete root;
		tempStream += e->getExpression();

		STRING DatasegVar = (SStk.top()).label;
		SStk.pop();

		tempStream += "\tPOP\tAX\n"
					  "\tMOV\t";
		tempStream += DatasegVar.c_str();
		tempStream += ", AX\n";

		DatasegVar += "\tDW\t0\n";
		DataSeg.push_back(DatasegVar);
	}
	else if (semanticToken == "@testifdefine")
	{
		if( findDSCPinglobal( (SStk.top()).label ))
			throw Except("ERR : ID IS ALREADY DEFINED -> " + (SStk.top()).label );
	}
	else if (semanticToken == "@newfunctionHascome")
	{
		int end = myframe.size();

		for(int i = 0;i < end;i++)
			delete myframe[i];

		myframe.resize(0);

		myargs = new ARGVECT;

		mybp = 0;
	}
	else if (semanticToken == "@funcdecempty")
	{
		myargs->push_back( (SStk.top()).prevtype);

		SStk.push( SSCell(""));

		(SStk.top()).prevtype = SIMPLE;
		(SStk.top()).columnnum = 1;
	}
	else if (semanticToken == "@funcframePUSHARG")
	{
		SSCell mytop = SStk.top();
		SStk.pop();

		STRING name = mytop.label;
		BasicT thetype = mytop.prevtype;
		int indim = mytop.columnnum;
		mybp++;

		switch(thetype)
		{
		case SIMPLE:
					myframe.push_back( new Simple( name, "", (SStk.top()).prevtype));
					break;
		case ARR1D:
					myframe.push_back( new OneDArr( name, "", (SStk.top()).prevtype));
					break;
		case ARR2D:
					myframe.push_back( new TwoDArr( name, "", indim, (SStk.top()).prevtype));
					break;
		default:
					throw Except("OMRAN!!!!!!");
					break;
		}

		SStk.pop();
	}
	else if (semanticToken == "@funcENDframe")
	{
		if( findwithdifferentargs( (SStk.top()).label))
			throw Except("ERR : FUNCTION HAS ALREADY DEFINRD WITH DIFFERENT NUMBER OF ARGUMENTS!!");

		funcprototypes.push_back( FuncCell((SStk.top()).prevtype, (SStk.top()).label, myargs));

		int end = myframe.size();

		for(int i = 0;i < end;i++)
			delete myframe[i];

		myframe.resize( 0);

		anotherStream += "__";
		anotherStream += (SStk.top()).label.c_str();
		anotherStream += "\tENDP\n\n";
			
		SStk.pop();
		SStk.pop();
	}
	else if (semanticToken == "@funcNOframe&GoON")
	{
		if( findwithdifferentargs( (SStk.top()).label))
			throw Except("ERR : FUNCTION HAS ALREADY DEFINRD WITH DIFFERENT NUMBER OF ARGUMENTS!!");

		funcprototypes.push_back( FuncCell((SStk.top()).prevtype, (SStk.top()).label, myargs));

		int end = myframe.size();

		for(int i = 0;i < end;i++)
			delete myframe[i];

		myframe.resize( 0);
		
		SStk.pop();
	}
	else if (semanticToken == "@funcframePUSHARGENDBP")
	{
		SSCell mytop = SStk.top();
		SStk.pop();

		STRING name = mytop.label;
		BasicT thetype = mytop.prevtype;
		int indim = mytop.columnnum;
		mybp++;

		switch(thetype)
		{
		case SIMPLE:
					myframe.push_back( new Simple( name, "", (SStk.top()).prevtype));
					break;
		case ARR1D:
					myframe.push_back( new OneDArr( name, "", (SStk.top()).prevtype));
					break;
		case ARR2D:
					myframe.push_back( new TwoDArr( name, "", indim, (SStk.top()).prevtype));
					break;
		default:
					throw Except("OMRAN!!!!!!");
					break;
		}

		SStk.pop();
		myframe.push_back( new Simple( "", "", INT));
	}
	else if (semanticToken == "@funcframeENDBP")
	{
		myframe.push_back( new Simple( "", "", INT));
	}		
	else if (semanticToken == "@localsDeclareid")
	{
		STRING myid = (SStk.top()).label;
		SStk.pop();
		STRING mycode = "[BP - ";
		mycode += int2String( 2 * (myframe.size() - mybp));
		mycode += "]";

		myframe.push_back( new Simple(myid, mycode, (SStk.top()).prevtype));
		SStk.pop();
	}
	else if (semanticToken == "@localsDeclareid&GOON")
	{
		STRING myid = (SStk.top()).label;
		SStk.pop();
		STRING mycode = "[BP - ";
		mycode += int2String( 2 * (myframe.size() - mybp));
		mycode += "]";

		myframe.push_back( new Simple(myid, mycode, (SStk.top()).prevtype));
	}
	else if (semanticToken == "@functionINITLOCAL")
	{
		if( findDSCPinframe( (SStk.top()).label ))
			throw Except("ERR : ID IS ALREADY DEFINED -> " + (SStk.top()).label );

		STRING myid = (SStk.top()).label;
		SStk.pop();
		STRING mycode = "[BP - ";
		mycode += int2String( 2 * (myframe.size() - mybp));
		mycode += "]";

		myframe.push_back( new Simple(myid, mycode, (SStk.top()).prevtype));

		SStk.push( SSCell(mycode));
	}
	else if (semanticToken == "@HOSSEIN@INITELOCAL")
	{
		//expressioncodder( functionStream);
		Expression* e = new Expression( root, this);
		delete root;
		functionStream += e->getExpression();

		STRING mycode = (SStk.top()).label;
		SStk.pop();

		functionStream += "\tPOP\tAX\n"
					     "\tMOV\t";
		functionStream += mycode.c_str();
		functionStream += ", AX\n";
	}
	else if (semanticToken == "@StrLitLOCAL")
	{
		STRING StrLit = nextLiteral();

		STRING mycode = (SStk.top()).label;
		SStk.pop();
		
		if((SStk.top()).prevtype != CHARSTAR)
			throw Except("ERR : A STRING CANNOT ASSIGN TO NONE CHAR * TYPE IDS");
		
		functionStream += "\tLEA\tAX, ";
		functionStream += StrLit.c_str();
		functionStream += "\n"
					  "\tMOV\t";
		functionStream += mycode.c_str();
		functionStream += ", AX\n";

		StrLit += "\tDB\t";
		StrLit += "\'";
		StrLit += STRING( lexeme, 1 , lexeme.size() - 2);
		StrLit += "\'";
		StrLit += "\nDB 0\n";
		DataSeg.push_back(StrLit);
	}
	else if (semanticToken == "@POPSSLOCAL")
	{
		SStk.pop();
	}
	else if (semanticToken == "@localsDeclareARRONE")
	{
		int firstdim = (SStk.top()).columnnum;
		SStk.pop();

		STRING myid = (SStk.top()).label;
		SStk.pop();
		STRING mycode = "[BP - ";
		mycode += int2String( 2 * ((myframe.size() - mybp - 1 ) + firstdim) );
		//mycode += "]";

		myframe.push_back( new OneDArr(myid, mycode, (SStk.top()).prevtype));
		SStk.pop();

		for(int i = 0;i < firstdim - 1;i++)
			myframe.push_back( new Simple("", "", INT));
	}
	else if (semanticToken == "@localsDeclareARRONE&GOON")
	{
		int firstdim = (SStk.top()).columnnum;
		SStk.pop();

		STRING myid = (SStk.top()).label;
		SStk.pop();
		STRING mycode = "[BP - ";
		mycode = int2String( 2 * ((myframe.size() - mybp - 1 ) + firstdim) );
		//mycode += "]";

		myframe.push_back( new OneDArr(myid, mycode, (SStk.top()).prevtype));

		for(int i = 0;i < firstdim - 1;i++)
			myframe.push_back( new Simple("", "", INT));
	}
	else if (semanticToken == "@localsDeclareARRTWO")
	{
		int seconddim = (SStk.top()).columnnum;
		SStk.pop();
		int firstdim = (SStk.top()).columnnum;
		SStk.pop();

		STRING myid = (SStk.top()).label;
		SStk.pop();
		STRING mycode = "[BP - ";
		mycode += int2String( 2 * ((myframe.size() - mybp - 1 ) + firstdim * seconddim));
		//mycode += "]";

		myframe.push_back( new TwoDArr(myid, mycode, seconddim, (SStk.top()).prevtype));
		SStk.pop();

		for(int i = 0;i < firstdim * seconddim - 1;i++)
			myframe.push_back( new Simple("", "", INT));
	}
	else if (semanticToken == "@localsDeclareARRTWO&GOON")
	{
		int seconddim = (SStk.top()).columnnum;
		SStk.pop();
		int firstdim = (SStk.top()).columnnum;
		SStk.pop();

		STRING myid = (SStk.top()).label;
		SStk.pop();
		STRING mycode = "[BP - ";
		mycode += int2String( 2 * ((myframe.size() - mybp - 1 ) + firstdim * seconddim));
		//mycode += "]";

		myframe.push_back( new TwoDArr(myid, mycode, seconddim, (SStk.top()).prevtype));

		for(int i = 0;i < firstdim * seconddim - 1;i++)
			myframe.push_back( new Simple("", "", INT));
	}
//ERERTGSDGDYGFKJUGCXDFHDFHKJGKJGLHGVFKJHVJHGOI:LKJCCHNHNUHUMH<MHYFTRDHGYXFHFXDGHFDGHHGGH FOHSHE BAD CTRL_G
	else if (semanticToken == "@makeCodeforframe")
	{
		DSCP* tempdscp;

		for(int i = 0;i < mybp;i++)
		{
			myframe[i]->code += "[BP + ";
			myframe[i]->code += int2String(2 * i + 4);
			myframe[i]->code += "]";
		}

		for(i = 0;i < mybp - 1;i++)
		{
			for(int j = 0;j < mybp - i - 1;j++)
			{
				tempdscp = myframe[j];
				myframe[j] = myframe[j + 1];
				myframe[j + 1] = tempdscp;
			}
		}

		anotherStream += "__";
		anotherStream += (SStk.top()).label.c_str();
		anotherStream += "\tPROC\tNEAR\n"
						 "\tPUSH\tBP\n"
						 "\tMOV\tBP, SP\n";

		if(myframe.size() - mybp - 1 != 0)
		{
			anotherStream += "\tSUB\tSP,\t";
			anotherStream += int2String( (myframe.size() - mybp - 1) * 2);
			anotherStream += "\n";
		}

		anotherStream += functionStream;
		functionStream.resize(0);
		anotherStream += "\t;-----------------------------------\n";
	}
//TYPE graph
	else if (semanticToken == "@TYPEPUSHINT")
	{
		SStk.push(SSCell( INT));
	}
	else if (semanticToken == "@TYPEPUSHCHAR")
	{
		SStk.push(SSCell( CHAR));
	}
	else if (semanticToken == "@TYPEPUSHSTAR")
	{
		switch((SStk.top()).prevtype)
		{
		case CHAR :
				(SStk.top()).prevtype = CHARSTAR;
				break;
		case INT :
				(SStk.top()).prevtype = INTSTAR;
				break;
		default:
				throw Except("ERR : UNALLOWEDED TYPE ON THE STACK");
				break;
		}
	}
//ID graph
	else if (semanticToken == "@idPushSSID")
	{
		if( searchforthisinfunctionframe( lexeme))
			throw Except("ERR : ID IS ALREADY DEFINED AS FUNCTION NAME -> " + lexeme );

		(SStk.top()).label = lexeme;
	}
	else if (semanticToken == "@idcorrect1Darr")
	{
		(SStk.top()).prevtype = ARR1D;
	}
	else if (semanticToken == "@idcorrect2Darr")
	{
		int mydim = (SStk.top()).columnnum;
		SStk.pop();
		(SStk.top()).prevtype = ARR2D;
	}
//ST graph
	else if (semanticToken == "@HOSSEIN@FuncRet")
	{
		//expressioncodder( anotherStream);
		Expression* e = new Expression( root, this);
		delete root;
		anotherStream += e->getExpression();

		anotherStream +="\tPOP\tAX\n";
		anotherStream += "\t;-----------------------------------\n"
						 "\tMOV\tSP,BP\n"
						 "\tPOP\tBP\n"
						 "\tRET\n";
	}
	else if (semanticToken == "@;POPER")
	{
		anotherStream += "\tPOP\tCX\n";
	}
	else if (semanticToken == "@ifCMPJP")
	{
		STRING StrLab = nextLabel();

		SStk.push(SSCell( StrLab));

		anotherStream +="\tPOP\tAX\n"
						"\tCMP\tAX,0\n"
						"\tJZ\t";
		anotherStream +=StrLab.c_str();
		anotherStream +='\n';
	}
	else if (semanticToken == "@ifCOMPJP")
	{
		STRING tempstr = (SStk.top()).label;
		SStk.pop();

		anotherStream += tempstr.c_str();
		anotherStream += ":\n";
	}
	else if (semanticToken == "@ifCOMPJPJP")
	{
		STRING StrLab = nextLabel();

		anotherStream += "\tJMP\t";
		anotherStream += StrLab.c_str();
		anotherStream += '\n';

		STRING tempstr = (SStk.top()).label;
		SStk.pop();

		anotherStream += tempstr.c_str();
		anotherStream += ":\n";

		SStk.push(SSCell( StrLab));
	}
	else if (semanticToken == "@whilePUSHADDR")
	{
		STRING StrLab = nextLabel();

		anotherStream += StrLab.c_str();
		anotherStream += ":\n";

		SStk.push(SSCell( StrLab));
	}
	else if (semanticToken == "@whileCMPJP")
	{
		STRING StrLab = nextLabel();

		SStk.push(SSCell( StrLab));

		anotherStream += "\tPOP\tAX\n"
						 "\tCMP\tAX,0\n"
						 "\tJZ\t";
		anotherStream += StrLab.c_str();
		anotherStream += '\n';
	}
	else if (semanticToken == "@whileJPCOMPJP")
	{
		STRING tempstr = (SStk.top()).label;
		SStk.pop();

		anotherStream += "\tJMP\t";
		anotherStream += (SStk.top()).label.c_str();
		anotherStream += '\n';

		SStk.pop();

		anotherStream += tempstr.c_str();
		anotherStream += ":\n";
	}
	else if (semanticToken == "@,POPER")
	{
		anotherStream += "\tPOP\tCX\n";
	}
	else if (semanticToken == "@forKEEP")
	{
		keep4later = root;
	}
	else if (semanticToken == "@forJPADDRPUSH")
	{
		STRING StrLab = nextLabel();
		STRING StrLab2 = nextLabel();

		anotherStream += "\tJMP\t";
		anotherStream += StrLab.c_str();
		anotherStream += '\n';

		anotherStream += StrLab2.c_str();
		anotherStream += ":\n";

		SStk.push(SSCell( StrLab2));
		SStk.push(SSCell( StrLab));
	}
	else if (semanticToken == "@forCOMPJP@keepHOSSEIN @JP")
	{
		anotherStream += "\tPOP\tCX\n";

		STRING tempstr = (SStk.top()).label;
		SStk.pop();

		anotherStream += tempstr.c_str();
		anotherStream += ":\n";

		//expressioncodder( anotherStream);
		Expression* e = new Expression( keep4later, this);
		delete keep4later;
		anotherStream += e->getExpression();

		STRING StrLab = nextLabel();

		SStk.push(SSCell( StrLab));

		anotherStream += "\tPOP\tAX\n"
					 	 "\tCMP\tAX,0\n"
						 "\tJZ\t";
		anotherStream += StrLab.c_str();
		anotherStream += '\n';
	}
	else if (semanticToken == "@forJPCOMPJP")
	{
		STRING tempstr = (SStk.top()).label;
		SStk.pop();

		anotherStream += "\tJMP\t";
		anotherStream += (SStk.top()).label.c_str();
		anotherStream += '\n';

		SStk.pop();

		anotherStream += tempstr.c_str(); 
		anotherStream += ":\n";
	}
	else if (semanticToken == "@HOSSEIN")
	{
		//expressioncodder( anotherStream);
		Expression* e = new Expression( root, this);
		delete root;
		anotherStream += e->getExpression();
	}
//E graph
	else if (semanticToken == "@Node>>=")
	{
		underconstruct = new Tree(">>=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node|=")
	{
		underconstruct = new Tree("|=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node/=")
	{
		underconstruct = new Tree("/=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node-=")
	{
		underconstruct = new Tree("-=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node=")
	{
		underconstruct = new Tree("=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node+=")
	{
		underconstruct = new Tree("+=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node*=")
	{
		underconstruct = new Tree("*=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node&=")
	{
		underconstruct = new Tree("&=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node<<=")
	{
		underconstruct = new Tree("<<=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@NodeComp")
	{
		underconstruct = TreeStk.top();
		TreeStk.pop();
		underconstruct->right = root;
		root = underconstruct;
	}
//E1 graph
	else if (semanticToken == "@Node||")
	{
		underconstruct = new Tree("||", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E2 graph
	else if (semanticToken == "@Node&&")
	{
		underconstruct = new Tree("&&", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E3 graph
	else if (semanticToken == "@Node|")
	{
		underconstruct = new Tree("|", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E4 graph
	else if (semanticToken == "@Node&")
	{
		underconstruct = new Tree("&", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E5 graph
	else if (semanticToken == "@Node==")
	{
		underconstruct = new Tree("==", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node!=")
	{
		underconstruct = new Tree("!=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E6 graph
	else if (semanticToken == "@Node<=")
	{
		underconstruct = new Tree("<=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node<")
	{
		underconstruct = new Tree("<", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node>")
	{
		underconstruct = new Tree(">", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node>=")
	{
		underconstruct = new Tree(">=", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E7 graph
	else if (semanticToken == "@Node<<")
	{
		underconstruct = new Tree("<<", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node>>")
	{
		underconstruct = new Tree(">>", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E8 graph
	else if (semanticToken == "@Node+")
	{
		underconstruct = new Tree("+", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node-")
	{
		underconstruct = new Tree("-", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E9 graph
	else if (semanticToken == "@Node*")
	{
		underconstruct = new Tree("*", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
	else if (semanticToken == "@Node/")
	{
		underconstruct = new Tree("/", 'O');
		underconstruct->left = root;
		TreeStk.push( underconstruct);
	}
//E10 graph
	else if (semanticToken == "@Node&U")
	{
		underconstruct = new Tree("&", 'O');
		underconstruct->right = root;
		root = underconstruct;
	}
	else if (semanticToken == "@Node--L")
	{
		underconstruct = new Tree("--", 'O');
		underconstruct->right = root;
		root = underconstruct;
	}
	else if (semanticToken == "@Node!")
	{
		underconstruct = new Tree("!", 'O');
		underconstruct->right = root;
		root = underconstruct;
	}
	else if (semanticToken == "@Node++L")
	{
		underconstruct = new Tree("++", 'O');
		underconstruct->right = root;
		root = underconstruct;
	}
	else if (semanticToken == "@Node*U")
	{
		underconstruct = new Tree("*", 'O');
		underconstruct->right = root;
		root = underconstruct;
	}
	else if (semanticToken == "@NumLit")
	{
		root = new Tree(lexeme, 'C');
	}
	else if (semanticToken == "@Nodeid")
	{
		root = new Tree(lexeme, 'I');
	}
	else if (semanticToken == "@Node--R")
	{
		if (root->left != NULL && root->right != NULL)
			throw Except("ERR : Right ++ cannot apply to none lvalues!!");

		root->right = new Tree("--", 'O');
	}
	else if (semanticToken == "@Node++R")
	{
		if (root->left != NULL && root->right != NULL)
			throw Except("ERR : Right ++ cannot apply to none lvalues!!");

		root->right = new Tree("++", 'O');
	}
	else if (semanticToken == "@NodeArray1D")
	{
		root->type = 'A';
		temproot = root;
		functionargs = new ARGLIST;
		temproot->arguments = functionargs;
	}
	else if (semanticToken == "@FirstDim")
	{
		functionargs->push_back(root);
		root = temproot;
	}
	else if (semanticToken == "@NodeArray2D")
	{
		temproot = root;
	}
	else if (semanticToken == "@SecondDim")
	{
		functionargs->push_back(root);
		root = temproot;
	}
	else if (semanticToken == "@NodeNewARGVECT")
	{
		root->type = 'F';
		temproot = root;
		functionargs = new ARGLIST;
	}
	else if (semanticToken == "@PushARG")
	{
		functionargs->push_back(root);
	}
	else if (semanticToken == "@NodeFinalFUNC")
	{
		root = temproot;
		root->arguments = functionargs;
	}
//ST PRINTS
	else if (semanticToken == "@PUSHPI&CALL")
	{
		anotherStream += "\tMOV\tAX, OFFSET _HNSI\n"
						 "\tPUSH\tAX\n"
						 "\tCALL\t_printf\n"
						 "\tPOP\tCX\n"
						 "\tPOP\tCX\n";
	}
	else if (semanticToken == "@PUSHPC&CALL")
	{
		anotherStream += "\tMOV\tAX, OFFSET _HNSC\n"
						 "\tPUSH\tAX\n"
						 "\tCALL\t_printf\n"
						 "\tPOP\tCX\n"
						 "\tPOP\tCX\n";
	}
	else if (semanticToken == "@PUSHPS*&CALL")
	{
		DSCP * tempDSCP = findDSCP( lexeme);

		if(tempDSCP == NULL)
			throw Except("VARIBAL NOT FOUND!");

		STRING tempname = tempDSCP->getname();

		if(tempname.size() == 0)
			throw Except("TOOL E MOTEGHAYYER KE 0 NEMISHE!");

		if(tempname[0] == '_')
		{
			anotherStream += "\tPUSH\tOFFSET ";
			anotherStream += tempname;
			anotherStream += '\n';
		}
		else
		{
			anotherStream += "\tPUSH\tWORD PTR ";
			anotherStream += tempDSCP->getcode();
			anotherStream += '\n';
		}

		anotherStream += "\tMOV\tAX, OFFSET _HNSS\n"
						 "\tPUSH\tAX\n"
						 "\tCALL\t_printf\n"
						 "\tPOP\tCX\n"
						 "\tPOP\tCX\n";
	}
	else if (semanticToken == "@PUSHPSL&CALL")
	{
		STRING StrLit = nextLiteral();

		anotherStream += "\tPUSH\tOFFSET ";
		anotherStream += StrLit + '\n';
		anotherStream += "\tMOV\tAX, OFFSET _HNSS\n"
						 "\tPUSH\tAX\n"
						 "\tCALL\t_printf\n"
						 "\tPOP\tCX\n"
						 "\tPOP\tCX\n";

		StrLit += "\tDB\t\'";
		StrLit += STRING( lexeme, 1, lexeme.size() - 2);
		StrLit += "\'\n\t\tDB\t0\n";
		DataSeg.push_back(StrLit);
	}
	else
	{
		throw Except("DADAM VAY REVAL MAFHOOMI");
	}
}
